"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import platform
import re
from logging import Logger
from typing import Union, overload, Literal

import semver

from Setter.Common.ControllerPins import ControllerPins
from Setter.Common.SpiConfig import SpiConfig
from Setter.Common.UartConfig import UartConfig
from Setter.Exceptions.BoardConfigNotFoundException import BoardConfigNotFoundException
from Setter.Exceptions.BoardNotFoundException import BoardNotFoundException
from Setter.Exceptions.SystemErrorException import SystemErrorException
from Setter.Logging import Logging


class Boards:
	"""
	Board pin configuration class.

	Methods
	-------
	get_model_config(model: str, config_type: str) -> Union[ControllerPins, SpiConfig, UartConfig]:
		Return interface configuration for specified board.
	get_controller_pins(model: str) -> ControllerPins:
		Return controller configuration for specified board.
	"""

	__models = {
		'IQD-GW-01': [
			{
				'controller': ControllerPins(2, 0, 1, 12, 11),
				'spi': SpiConfig('/dev/spidev1.0', 19, 3, 10, True),
				'uart': UartConfig('/dev/ttyS1', 19, -1, 10, 57600, True),
				'kernel_versions': [],
			},
		],
		'IQD-GW-02': [
			{
				'controller': ControllerPins(2, 0, 1, 12, 11),
				'spi': SpiConfig('/dev/spidev1.0', 19, 3, -1, True, 7, 10, 6),
				'uart': UartConfig('/dev/ttyS1', 19, 3, -1, 57600, True, 7, 10, 6),
				'kernel_versions': [],
			},
		],
		'IQD-GW04': [
			{
				'controller': ControllerPins(25, 27, 17, 16, -1),
				'spi': SpiConfig('/dev/spidev1.0', 23, 22, -1, True, 18, 7, 6),
				'uart': UartConfig('/dev/ttyAMA1', 23, -1, -1, 57600, True, 18, 7, 6),
				'kernel_versions': ['<6.0.0'],
			},
			{
				'controller': ControllerPins(25, 27, 17, 16, -1),
				'spi': SpiConfig('/dev/spidev1.0', 23, 22, -1, True, 18, 7, 6),
				'uart': UartConfig('/dev/ttyAMA3', 23, -1, -1, 57600, True, 18, 7, 6),
				'kernel_versions': ['>=6.0.0', '<6.6.0'],
			},
			{
				'controller': ControllerPins(537, 539, 529, 528, -1),
				'spi': SpiConfig('/dev/spidev1.0', 535, 534, -1, True, 530, 519, 518),
				'uart': UartConfig('/dev/ttyAMA3', 535, -1, -1, 57600, True, 530, 519, 518),
				'kernel_versions': ['>=6.6.0'],
			},
		],
		'Unipi Iris Zulu': [
			{
				'controller': ControllerPins(504, 507, 506),
				'spi': SpiConfig('/dev/spidev1.0', 511, 510, -1, True, 509, 508, 505),
				'uart': UartConfig('/dev/ttymxc3', 511, 510, -1, 57600, True, 509, 508, 505),
				'kernel_versions': [],
			},
		],
	}
	__logger: Logger = Logging.get_logger()

	@overload
	@classmethod
	def get_model_config(cls, model: str, config_type: Literal['controller']) -> ControllerPins:
		...

	@overload
	@classmethod
	def get_model_config(cls, model: str, config_type: Literal['spi']) -> SpiConfig:
		...

	@overload
	@classmethod
	def get_model_config(cls, model: str, config_type: Literal['uart']) -> UartConfig:
		...

	@classmethod
	def get_model_config(cls, model: str, config_type: str) -> Union[ControllerPins, SpiConfig, UartConfig]:
		"""
		Return interface configuration for specified board.

		Parameters
		----------
		model: str
			board model
		config_type: str
			configuration interface type

		Returns
		-------
		Union[SpiConfig, UartConfig]
			interface configuration object

		Raises
		------
		BoardNotFoundException
			raised if specified board model is unknown
		BoardConfigNotFoundException
			raised if specified board does not have configuration for the required interface
		SystemErrorException
			raised if kernel version cannot be parsed
		"""
		if model.startswith('IQD-GW-02'):
			model = 'IQD-GW-02'
		elif model.startswith('IQD-GW04'):
			model = 'IQD-GW04'
		if model not in cls.__models:
			error_str = f'Board model {model} not found.'
			cls.__logger.error(error_str)
			raise BoardNotFoundException(error_str)
		current_kernel: semver.VersionInfo = cls._get_kernel_version()
		for config in cls.__models[model]:
			matches = True
			for version in config['kernel_versions']:
				if not current_kernel.match(version):
					matches = False
					break
			if matches and config_type in config:
				return config[config_type]
		error_str = f'Board model {model} {config_type} pin configuration for kernel {current_kernel} not found.'
		cls.__logger.error(error_str)
		raise BoardConfigNotFoundException(error_str)

	@classmethod
	def get_controller_pins(cls, model: str) -> ControllerPins:
		"""
		Return controller configuration for specified board.

		Parameters
		----------
		model: str
			board model

		Returns
		-------
		ControllerPins
			controller pins configuration object

		Raises
		------
		BoardNotFoundException
			raised if specified board model is unknown
		BoardConfigNotFoundException
			raised if specified board does not have controller pin configuration
		SystemErrorException
			raised if kernel version cannot be parsed
		"""
		return cls.get_model_config(model, 'controller')

	@staticmethod
	def _get_kernel_version() -> semver.VersionInfo:
		"""
		Return kernel version.

		Returns
		-------
		semver.VersionInfo
			kernel version

		Raises
		------
		SystemErrorException
			raised if kernel version cannot be parsed
		"""
		kernel_version = re.search(r'\d+\.\d+\.\d+', platform.release())
		if kernel_version is None:
			raise SystemErrorException('Failed to parse kernel version.')
		return semver.VersionInfo.parse(kernel_version.group())
