"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import errno
import os
import sys

from Setter.Exceptions.IOErrorException import IOErrorException

if sys.version_info >= (3, 8):
	import shutil
else:
	from distutils.errors import DistutilsFileError
	from distutils.dir_util import remove_tree, copy_tree
	from distutils.file_util import copy_file


class FileManager:
	"""
	File manager class.

	Methods
	-------
	copy(source: str, destination: str) -> None:
		Copy a file.
	copy_tree(source: str, destination: str) -> None:
		Recursively copy a directory.
	mkdir(path: str) -> None:
		Create a directory.
	remove(path: str) -> None:
		Remove a file.
	remove_tree(path: str) -> None:
		Recursively remove a directory.
	symlink(destination: str, link: str, override: bool) -> None:
		Create a symbolic link.
	"""

	@staticmethod
	def copy(source: str, destination: str) -> None:
		"""
		Copy a file.

		Parameters
		----------
		source: str
			Path to the source file
		destination: str
			Path to the destination file

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the file cannot be copied
		"""
		if os.path.exists(destination):
			os.remove(destination)
		try:
			if sys.version_info >= (3, 8):
				shutil.copy2(
					src=source,
					dst=destination,
				)
			else:
				copy_file(
					src=source,
					dst=destination,
					preserve_mode=True,
					preserve_times=True,
					update=True,
				)
		except (shutil.Error, DistutilsFileError, OSError) as error:
			raise IOErrorException(str(error)) from error

	@staticmethod
	def copy_tree(source: str, destination: str) -> None:
		"""
		Recursively copy a directory.

		Parameters
		----------
		source: str
			Path to the source directory
		destination: str
			Path to the destination directory

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the directory cannot be copied
		"""
		FileManager.remove_tree(destination)
		FileManager.mkdir(destination)
		try:
			if sys.version_info >= (3, 8):
				shutil.copytree(
					src=source,
					dst=destination,
					symlinks=True,
					dirs_exist_ok=True,
				)
			else:
				copy_tree(
					src=source,
					dst=destination,
					preserve_symlinks=True,
					preserve_mode=True,
					preserve_times=True,
					update=True,
				)
		except (shutil.Error, DistutilsFileError) as error:
			raise IOErrorException(str(error)) from error

	@staticmethod
	def mkdir(path: str) -> None:
		"""
		Create a directory.

		Parameters
		----------
		path: str
			Path to the directory

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the directory cannot be created
		"""
		try:
			os.makedirs(path, exist_ok=True)
		except OSError as error:
			raise IOErrorException(str(error)) from error

	@staticmethod
	def remove(path: str) -> None:
		"""
		Remove a file.

		Parameters
		----------
		path: str
			Path to the file

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the file cannot be removed
		"""
		if not os.path.exists(path):
			return
		try:
			os.remove(path)
		except OSError as error:
			raise IOErrorException(str(error)) from error

	@staticmethod
	def remove_tree(path: str) -> None:
		"""
		Recursively remove a directory.

		Parameters
		----------
		path: str
			Path to the directory

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the directory cannot be removed
		"""
		if not os.path.exists(path):
			return
		try:
			if os.path.isdir(path):
				if sys.version_info >= (3, 8):
					shutil.rmtree(path=path)
				else:
					remove_tree(directory=path)
			else:
				os.unlink(path)
		except (shutil.Error, DistutilsFileError, IOError) as error:
			raise IOErrorException(str(error)) from error

	@staticmethod
	def symlink(destination: str, link: str, override: bool) -> None:
		"""
		Create a symbolic link.

		Parameters
		----------
		destination: str
			source file for symbolic link
		link: str
			symbolic link name / path
		override: bool
			overwrite existing symbolic link

		Returns
		-------
		None

		Raises
		------
		IOErrorException
			raised if the symbolic link cannot be created
		"""
		try:
			os.symlink(destination, link)
		except OSError as os_error:
			if os_error.errno == errno.EEXIST and override:
				try:
					os.remove(link)
					os.symlink(destination, link)
				except OSError as os_error_new:
					raise IOErrorException(str(os_error_new)) from os_error_new
			raise IOErrorException(str(os_error)) from os_error
