"""
Copyright 2021-2024 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import re

from packaging import version
from packaging.version import Version


class GatewayImage:
	"""Gateway image class."""

	product: str
	os_type: str
	os_version: Version

	def __init__(self, product: str, os_type: str, os_version: Version):
		"""
		Initialize GatewayImage object.

		Parameters
		----------
		product: str
			Product name
		os_type: str
			Operating system type
		os_version: Version
			Operating system version
		"""
		self.product = product
		self.os_type = os_type
		self.os_version = os_version

	def __eq__(self, other):
		"""
		Check if gateway image object is equal to other gateway image object.

		Parameters
		----------
		other: GatewayImage
			Other gateway image object

		Returns
		-------
		bool
			True if gateway image object is equal to other gateway image object, False otherwise
		"""
		return self.product == other.product \
			and self.os_type == other.os_type \
			and self.os_version == other.os_version

	def __repr__(self):
		"""
		Return string representation of gateway image object.

		Returns
		-------
		str
			String representation of gateway image object
		"""
		return str(self)

	def __str__(self):
		"""
		Return string representation of gateway image object.

		Returns
		-------
		str
			String representation of gateway image object
		"""
		major, minor, patch = self.os_version.release
		os_version: str = f'v{major}.{minor}.{patch}'
		pre = self.os_version.pre
		if pre is not None:
			if pre[0] == 'a':
				os_version += f'-alpha{pre[1]}'
			elif pre[0] == 'b':
				os_version += f'-beta{pre[1]}'
			elif pre[0] == 'rc':
				os_version += f'-rc{pre[1]}'
		return f"{self.product}-{self.os_type}-{os_version}"

	@classmethod
	def from_str(cls, image: str):
		"""
		Create gateway image info object from string representation.

		Parameters
		----------
		image: str
			Gateway image string

		Returns
		-------
		GatewayImage
			Gateway image info object

		Raises
		------
		ValueError
			raised if image string is invalid
		"""
		pattern = '(?P<product>industrial|iqaros|iqube)-(?P<variant>armbian|yocto)-(?P<version>v\\d+\\.\\d+\\.\\d+(-(alpha|beta|rc\\d+))?)'
		matches = re.search(pattern, image)
		if not matches:
			raise ValueError(f"Invalid image version: {image}")
		return cls(matches.group('product'), matches.group('variant'), version.parse(matches.group('version')))
